@@ -4,6 +4,8 @@
 GCP resources.
 """
 
+import os
+
 from concurrent.futures import TimeoutError
 from google.cloud import bigquery, pubsub_v1, storage
 from google.cloud.logging_v2.logger import Logger
@@ -13,7 +15,13 @@ import json
 import pandas as pd
 from typing import Callable, List, Optional, Union
 
-pgb_project_id = 'ardent-cycling-243415'
+
+# get project id from environment variable, else default to production project
+# cloud functions use GCP_PROJECT
+if "GCP_PROJECT" in os.environ:
+    project_id_default = os.getenv("GCP_PROJECT")
+else:
+    project_id_default = os.getenv("GOOGLE_CLOUD_PROJECT", "ardent-cycling-243415")
 
 
 # --- Pub/Sub --- #
@@ -21,7 +29,7 @@ def publish_pubsub(
     topic_name: str,
     message: Union[bytes, dict],
     project_id: Optional[str] = None,
-    attrs: dict = {},
+    attrs: Optional[dict] = None,
     publisher: Optional[pubsub_v1.PublisherClient] = None
 ) -> str:
     """Publish messages to a Pub/Sub topic.
@@ -35,7 +43,7 @@ def publish_pubsub(
         message: The message to be published.
 
         project_id: GCP project ID for the project containing the topic.
-                    If None, the environment variable GOOGLE_CLOUD_PROJECT will be used.
+                    If None, the module's `project_id_default` will be used.
 
         attrs: Message attributes to be published.
 
@@ -51,17 +59,17 @@ def publish_pubsub(
         published message ID
     """
     if project_id is None:
-        project_id = pgb_project_id
-
+        project_id = project_id_default
     if publisher is None:
         publisher = pubsub_v1.PublisherClient()
+    if attrs is None:
+        attrs = {}
 
-    if type(message) == dict:
+    # enforce bytes type for message
+    if isinstance(message, dict):
         message = json.dumps(message).encode('utf-8')
-    try:
-        assert type(message) == bytes
-    except AssertionError:
-        raise ValueError('`message` must be bytes or a dict.')
+    if not isinstance(message, bytes):
+        raise TypeError('`message` must be bytes or a dict.')
 
     topic_path = publisher.topic_path(project_id, topic_name)
 
@@ -90,14 +98,14 @@ def pull_pubsub(
         max_messages: The maximum number of messages to pull.
 
         project_id: GCP project ID for the project containing the subscription.
-                    If None, the module's `pgb_project_id` will be used.
+                    If None, the module's `project_id_default` will be used.
 
         msg_only: Whether to work with and return the message contents only
                   or the full packet.
                   If `return_count` is True, it supersedes the returned object.
 
         callback: Function used to process each message.
-                  It's input type is determined by the value of `msg_only`.
+                  Its input type is determined by the value of `msg_only`.
                   It should return True if the message should be acknowledged,
                   else False.
 
@@ -108,7 +116,7 @@ def pull_pubsub(
         A list of messages
     """
     if project_id is None:
-        project_id = pgb_project_id
+        project_id = project_id_default
 
     # setup for pull
     subscriber = pubsub_v1.SubscriberClient()
@@ -122,7 +130,7 @@ def pull_pubsub(
     with subscriber:
 
         # pull
-        response = subscriber.pull(**request)
+        response = subscriber.pull(request)
 
         # unpack the messages
         message_list, ack_ids = [], []
@@ -153,7 +161,7 @@ def pull_pubsub(
                 "subscription": subscription_path,
                 "ack_ids": ack_ids,
             }
-            subscriber.acknowledge(**ack_request)
+            subscriber.acknowledge(ack_request)
 
     if not return_count:
         return message_list
@@ -166,7 +174,8 @@ def streamingPull_pubsub(
     callback: Callable[[PubsubMessage], None],
     project_id: str = None,
     timeout: int = 10,
-    block: bool = True
+    block: bool = True,
+    flow_control: Optional[dict] = None,
 ) -> Union[None, StreamingPullFuture]:
     """Pull and process Pub/Sub messages continuously in a background thread.
 
@@ -181,9 +190,10 @@ def streamingPull_pubsub(
                   acknowledgement logic.
 
         project_id: GCP project ID for the project containing the subscription.
-                    If None, the environment variable GOOGLE_CLOUD_PROJECT will be used.
+                    If None, the module's `project_id_default` will be used.
 
-        timeout: The number of seconds before the `subscribe` call times out.
+        timeout: The number of seconds before the `subscribe` call times out and
+                 closes the connection.
 
         block: Whether to block while streaming messages or return the
                StreamingPullFuture object for the user to manage separately.
@@ -196,14 +206,17 @@ def streamingPull_pubsub(
         or timeout.
     """
     if project_id is None:
-        project_id = pgb_project_id
+        project_id = project_id_default
+
+    if flow_control is None:
+        flow_control = {}
 
     subscriber = pubsub_v1.SubscriberClient()
     subscription_path = subscriber.subscription_path(project_id, subscription_name)
 
     # start receiving and processing messages in a background thread
     streaming_pull_future = subscriber.subscribe(
-        subscription_path, callback
+        subscription_path, callback, flow_control=flow_control
     )
 
     if block:
@@ -220,7 +233,11 @@ def streamingPull_pubsub(
 
 
 # --- BigQuery --- #
-def insert_rows_bigquery(table_id: str, rows: List[dict]):
+def insert_rows_bigquery(
+    table_id: str,
+    rows: List[dict],
+    project_id: Optional[str] = None,
+):
     """Insert rows into a table using the streaming API.
 
     Args:
@@ -229,8 +246,13 @@ def insert_rows_bigquery(table_id: str, rows: List[dict]):
         rows:       Data to load in to the table. Keys must include all required
                     fields in the schema. Keys which do not correspond to a
                     field in the schema are ignored.
+        project_id: GCP project ID for the project containing the topic.
+                    If None, the module's `project_id_default` will be used.
     """
-    bq_client = bigquery.Client(project=pgb_project_id)
+    if project_id is None:
+        project_id = project_id_default
+
+    bq_client = bigquery.Client(project=project_id)
     table = bq_client.get_table(table_id)
     errors = bq_client.insert_rows(table, rows)
     return errors
@@ -239,6 +261,7 @@ def insert_rows_bigquery(table_id: str, rows: List[dict]):
 def load_dataframe_bigquery(
     table_id: str,
     df: pd.DataFrame,
+    project_id: Optional[str] = None,
     use_table_schema: bool = True,
     logger: Optional[Logger] = None,
 ):
@@ -249,16 +272,20 @@ def load_dataframe_bigquery(
             {dataset}.{table}. For example, 'ztf_alerts.alerts'.
         df: Data to load in to the table. If the  dataframe schema does not match the
             BigQuery table schema, must pass a valid `schema`.
+        project_id: GCP project ID for the project containing the topic.
+                    If None, the module's `project_id_default` will be used.
         use_table_schema: Conform the dataframe to the table schema by converting
                           dtypes and dropping extra columns.
         logger: If not None, messages will be sent to the logger. Else, print them.
     """
     # setup
-    bq_client = bigquery.Client(project=pgb_project_id)
+    if project_id is None:
+        project_id = project_id_default
+    bq_client = bigquery.Client(project=project_id)
     table = bq_client.get_table(table_id)
 
     if use_table_schema:
-        d = df.reset_index()
+        my_df = df.reset_index()
 
         # set a job_config; bigquery will try to convert df.dtypes to match table schema
         job_config = bigquery.LoadJobConfig(schema=table.schema)
@@ -266,12 +293,12 @@ def load_dataframe_bigquery(
         # make sure the df has the correct columns
         bq_col_names = [s.name for s in table.schema]
         # pad missing columns
-        missing = [c for c in bq_col_names if c not in d.columns]
+        missing = [c for c in bq_col_names if c not in my_df.columns]
         for col in missing:
-            d[col] = None
+            my_df[col] = None
         # drop extra columns
-        dropped = list(set(d.columns) - set(bq_col_names))  # grab so we can report
-        d = d[bq_col_names]
+        dropped = list(set(my_df.columns) - set(bq_col_names))  # grab so we can report
+        my_df = my_df[bq_col_names]
         # tell the user what happened
         if len(dropped) > 0:
             msg = f'Dropping columns not in the table schema: {dropped}'
@@ -281,11 +308,11 @@ def load_dataframe_bigquery(
                 print(msg)
 
     else:
-        d = df
+        my_df = df
         job_config = None
 
     # load the data
-    job = bq_client.load_table_from_dataframe(d, table_id, job_config=job_config)
+    job = bq_client.load_table_from_dataframe(my_df, table_id, job_config=job_config)
     job.result()  # Wait for the job to complete.
 
     # report the results
@@ -300,31 +327,47 @@ def load_dataframe_bigquery(
         print(msg)
 
 
-
-def query_bigquery(query: str, project_id: Optional[str] = None):
+def query_bigquery(
+    query: str,
+    project_id: Optional[str] = None,
+    job_config: Optional[bigquery.job.QueryJobConfig] = None,
+) -> bigquery.job.QueryJob:
     """Query BigQuery.
 
+    Args:
+        query:
+            SQL query statement.
+        project_id:
+            The GCP project id that will be used to make the API call.
+            If not provided, the Pitt-Google production project id will be used.
+        job_config:
+            Optional job config to send with the query.
+
     Example query:
         ``
         query = (
             f'SELECT * '
-            f'FROM `{project_id}.{dataset}.{table}` '
+            f'FROM `{dataset_project_id}.{dataset}.{table}` '
             f'WHERE objectId={objectId} '
         )
         ``
 
-    Example of working with the query_job:
+    Examples of working with the query_job:
         ``
+        # Cast it to a DataFrame:
+        query_job.to_dataframe()
+
+        # Iterate row-by-row
         for r, row in enumerate(query_job):
             # row values can be accessed by field name or index
             print(f"objectId={row[0]}, candid={row['candid']}")
         ``
     """
     if project_id is None:
-        project_id = pgb_project_id
+        project_id = project_id_default
 
     bq_client = bigquery.Client(project=project_id)
-    query_job = bq_client.query(query)
+    query_job = bq_client.query(query, job_config=job_config)
 
     return query_job
 
@@ -340,8 +383,8 @@ def cs_download_file(localdir: str, bucket_id: str, filename: Optional[str] = No
         filename:   Name or prefix of the file(s) in the bucket to download.
     """
     # connect to the bucket and get an iterator that finds blobs in the bucket
-    storage_client = storage.Client(pgb_project_id)
-    bucket_name = f'{pgb_project_id}-{bucket_id}'
+    storage_client = storage.Client(project_id_default)
+    bucket_name = f'{project_id_default}-{bucket_id}'
     print(f'Connecting to bucket {bucket_name}')
     bucket = storage_client.get_bucket(bucket_name)
     blobs = storage_client.list_blobs(bucket, prefix=filename)  # iterator
@@ -367,8 +410,8 @@ def cs_upload_file(local_file: str, bucket_id: str, bucket_filename: Optional[st
         bucket_filename = local_file.split('/')[-1]
 
     # connect to the bucket
-    storage_client = storage.Client(pgb_project_id)
-    bucket_name = f'{pgb_project_id}-{bucket_id}'
+    storage_client = storage.Client(project_id_default)
+    bucket_name = f'{project_id_default}-{bucket_id}'
     print(f'Connecting to bucket {bucket_name}')
     bucket = storage_client.get_bucket(bucket_name)
 
